@@ -28,8 +28,7 @@
 
 void (*EXPORT_VAR (grub_grubnet_fini)) (void);
 
-grub_file_filter_t grub_file_filters_all[GRUB_FILE_FILTER_MAX];
-grub_file_filter_t grub_file_filters_enabled[GRUB_FILE_FILTER_MAX];
+grub_file_filter_t grub_file_filters[GRUB_FILE_FILTER_MAX];
 
 /* Get the device part of the filename NAME. It is enclosed by parentheses.  */
 char *
@@ -59,14 +58,17 @@ grub_file_get_device_name (const char *name)
 }
 
 grub_file_t
-grub_file_open (const char *name)
+grub_file_open (const char *name, enum grub_file_type type)
 {
   grub_device_t device = 0;
   grub_file_t file = 0, last_file = 0;
   char *device_name;
-  char *file_name;
+  const char *file_name;
   grub_file_filter_id_t filter;
 
+  /* Reset grub_errno before we start. */
+  grub_errno = GRUB_ERR_NONE;
+
   device_name = grub_file_get_device_name (name);
   if (grub_errno)
     goto fail;
@@ -76,10 +78,11 @@ grub_file_open (const char *name)
   if (file_name)
     file_name++;
   else
-    file_name = (char *) name;
+    file_name = name;
 
   device = grub_device_open (device_name);
   grub_free (device_name);
+  device_name = NULL;
   if (! device)
     goto fail;
 
@@ -89,7 +92,16 @@ grub_file_open (const char *name)
 
   file->device = device;
 
-  if (device->disk && file_name[0] != '/')
+  /* In case of relative pathnames and non-Unix systems (like Windows)
+   * name of host files may not start with `/'. Blocklists for host files
+   * are meaningless as well (for a start, host disk does not allow any direct
+   * access - it is just a marker). So skip host disk in this case.
+   */
+  if (device->disk && file_name[0] != '/'
+#if defined(GRUB_UTIL) || defined(GRUB_MACHINE_EMU)
+      && grub_strcmp (device->disk->name, "host")
+#endif
+     )
     /* This is a block list.  */
     file->fs = &grub_fs_blocklist;
   else
@@ -99,25 +111,31 @@ grub_file_open (const char *name)
 	goto fail;
     }
 
-  if ((file->fs->open) (file, file_name) != GRUB_ERR_NONE)
+  if ((file->fs->fs_open) (file, file_name) != GRUB_ERR_NONE)
     goto fail;
 
-  for (filter = 0; file && filter < ARRAY_SIZE (grub_file_filters_enabled);
+  file->name = grub_strdup (name);
+  grub_errno = GRUB_ERR_NONE;
+
+  for (filter = 0; file && filter < ARRAY_SIZE (grub_file_filters);
        filter++)
-    if (grub_file_filters_enabled[filter])
+    if (grub_file_filters[filter])
       {
 	last_file = file;
-	file = grub_file_filters_enabled[filter] (file);
+	file = grub_file_filters[filter] (file, type);
+	if (file && file != last_file)
+	  {
+	    file->name = grub_strdup (name);
+	    grub_errno = GRUB_ERR_NONE;
+	  }
       }
   if (!file)
     grub_file_close (last_file);
-    
-  grub_memcpy (grub_file_filters_enabled, grub_file_filters_all,
-	       sizeof (grub_file_filters_enabled));
 
   return file;
 
  fail:
+  grub_free (device_name);
   if (device)
     grub_device_close (device);
 
@@ -125,16 +143,17 @@ grub_file_open (const char *name)
 
   grub_free (file);
 
-  grub_memcpy (grub_file_filters_enabled, grub_file_filters_all,
-	       sizeof (grub_file_filters_enabled));
-
   return 0;
 }
 
+grub_disk_read_hook_t grub_file_progress_hook;
+
 grub_ssize_t
 grub_file_read (grub_file_t file, void *buf, grub_size_t len)
 {
   grub_ssize_t res;
+  grub_disk_read_hook_t read_hook;
+  void *read_hook_data;
 
   if (file->offset > file->size)
     {
@@ -155,7 +174,17 @@ grub_file_read (grub_file_t file, void *buf, grub_size_t len)
 
   if (len == 0)
     return 0;
-  res = (file->fs->read) (file, buf, len);
+  read_hook = file->read_hook;
+  read_hook_data = file->read_hook_data;
+  if (!file->read_hook)
+    {
+      file->read_hook = grub_file_progress_hook;
+      file->read_hook_data = file;
+      file->progress_offset = file->offset;
+    }
+  res = (file->fs->fs_read) (file, buf, len);
+  file->read_hook = read_hook;
+  file->read_hook_data = read_hook_data;
   if (res > 0)
     file->offset += res;
 
@@ -165,11 +194,12 @@ grub_file_read (grub_file_t file, void *buf, grub_size_t len)
 grub_err_t
 grub_file_close (grub_file_t file)
 {
-  if (file->fs->close)
-    (file->fs->close) (file);
+  if (file->fs->fs_close)
+    (file->fs->fs_close) (file);
 
   if (file->device)
     grub_device_close (file->device);
+  grub_free (file->name);
   grub_free (file);
   return grub_errno;
 }
@@ -185,9 +215,9 @@ grub_file_seek (grub_file_t file, grub_off_t offset)
 		  N_("attempt to seek outside of the file"));
       return -1;
     }
-  
+
   old = file->offset;
   file->offset = offset;
-    
+
   return old;
 }
